@@ -1,3 +1,11 @@
+# helper function:
+.as.units = function(x, value, dim) {
+  if (missing(dim))
+    structure(x, units = value, class = "units")
+  else
+    structure(x, units = value, dim = dim, class = "units")
+}
+
 #' Unit creation
 #'
 #' A number of functions are provided for creating unit objects. 
@@ -29,10 +37,11 @@
 #' units(x) <- "m/s"  # meters / second
 #' 
 #' # Alternatively, the easiest pipe-friendly way to set units:
-#' if(require(magrittr)) 
+#' if(requireNamespace("magrittr", quietly = TRUE)) {
+#'   library(magrittr)
 #'   y %>% set_units(m/s)
-#' 
-#' 
+#' } 
+#'
 #' # these are different ways of creating the same unit:
 #' # meters per second squared, i.e, acceleration
 #' x1 <- make_units(m/s^2)
@@ -110,10 +119,11 @@
 #'
 #' ## set_units()
 #' # set_units is a pipe friendly version of `units<-`. 
-#' if (require(magrittr)) {
-#'  1:5 %>% set_units(N/m^2)
-#'  # first sets to m, then converts to km
-#'  1:5 %>% set_units(m) %>% set_units(km)
+#' if(requireNamespace("magrittr", quietly = TRUE)) {
+#'   library(magrittr)
+#'   1:5 %>% set_units(N/m^2)
+#'   # first sets to m, then converts to km
+#'   1:5 %>% set_units(m) %>% set_units(km)
 #' }
 #' 
 #' # set_units has two modes of operation. By default, it operates with 
@@ -132,6 +142,8 @@
 #' # To remove units use
 #' units(x) <- NULL
 #' # or
+#' set_units(x, NULL)
+#' # or
 #' drop_units(y)
 make_units <- function(bare_expression, check_is_valid = TRUE) {
   as_units.call(substitute(bare_expression), check_is_valid = check_is_valid)
@@ -228,21 +240,29 @@ as_units.character <- function(x,
     implicit_exponents <- are_exponents_implicit(x)
   
   if(implicit_exponents)
-    return(.parse_unit_with_implicit_exponents(x))
+    x <- convert_implicit_to_explicit_exponents(x)
   
   x <- backtick(x)
   o <- try(expr <- parse(text = x)[[1]], silent = TRUE)
   
   if(inherits(o, "try-error")) {
-    warning("Could not parse expression: ", sQuote(x), 
-      ". Returning as a single symbolic unit()", call. = FALSE)
-    return(symbolic_unit(x, check_is_valid = check_is_valid))
+    warning("Could not parse expression: ", sQuote(x),          # nocov
+      ". Returning as a single symbolic unit()", call. = FALSE) # nocov
+    return(symbolic_unit(x, check_is_valid = check_is_valid))   # nocov
   }
 
   as_units.call(expr, check_is_valid = check_is_valid)
 }
 
 
+convert_implicit_to_explicit_exponents <- function(x) {
+  if (length(grep(c("[*/]"), x)) > 0)
+    stop("If 'implicit_exponents = TRUE', strings cannot contain `*' or `/'")
+  x <- gsub("\\b([^\\d-]+)([-]?\\d+)\\b", "\\1^(\\2)", x, perl =TRUE)
+  x <- gsub("\\s+", " * ", trimws(x), perl = TRUE)
+  x
+}
+
 #  ----- as_units.call helpers ------ 
 
 # from package:yasp, paste collapse with serial (oxford) comma
@@ -259,7 +279,7 @@ pc_and <- function(..., sep = "") {
     paste0( paste0(x[-lx], collapse = ", "), ", and ", x[lx])
 }
 
-`%not_in%` <- function(x, table) match(x, table, nomatch = 0L) == 0L
+#`%not_in%` <- function(x, table) match(x, table, nomatch = 0L) == 0L
 
 .msg_units_not_recognized <- function(unrecognized_symbols, full_expr) {
     
@@ -314,8 +334,11 @@ as_units.call <- function(x, check_is_valid = TRUE, ...) {
   
   if(missing(x) || identical(x, quote(expr =)) || 
      identical(x, 1) || identical(x, 1L))
-    return(structure(1, units = unitless, class = "units"))
+    return(.as.units(1, unitless))
   
+  if (is.vector(x) && any(is.na(x)))
+  	stop("a missing value for units is not allowed")
+
   stopifnot(is.language(x))
   
   vars <- all.vars(x)
@@ -348,7 +371,7 @@ See ?as_units for usage examples.")
 #"In ", sQuote(deparse(x)), " the numeric multiplier ", sQuote(as.numeric(unit)), " is invalid. 
 #Use `install_conversion_constant()` to define a new unit that is a multiple of another unit.")
   
-  structure(as.numeric(unit), units = units(unit), class = "units")
+  .as.units(as.numeric(unit), units(unit))
 }
 
 
@@ -375,20 +398,39 @@ symbolic_unit <- function(chr, check_is_valid = TRUE) {
       chr <- sym
   }
   
-  structure(1, units = .symbolic_units(chr), class = "units")
+  .as.units(1, .symbolic_units(chr))
 }
 
 
-#' drop units
+#' Drop Units
 #' 
-#' @param x a units object
+#' Drop units attribute and class.
+#' 
+#' @param x an object with units metadata.
 #' 
 #' @return the numeric without any units attributes, while preserving other
-#'   attributes like dimensions or other classes.
+#' attributes like dimensions or other classes.
 #'   
-#' @note Equivalent to \code{units(x) <- NULL}
+#' @details Equivalent to \code{units(x) <- NULL}, or the pipe-friendly version
+#' \code{set_units(x, NULL)}, but \code{drop_units} will fail if the object has
+#' no units metadata. Use the alternatives if you want this operation to succeed
+#' regardless of the object type.
 #' 
 #' @export
+#' @examples
+#' x <- 1
+#' y <- set_units(x, m/s)
+#' 
+#' # this succeeds
+#' drop_units(y)
+#' set_units(y, NULL)
+#' set_units(x, NULL)
+#' 
+#' \dontrun{
+#' # this fails
+#' drop_units(x)
+#' }
+#' 
 drop_units <- function(x) UseMethod("drop_units")
 
 #' @export
